@@ -21,7 +21,7 @@
  *          Robert Mibus
  */
 
-/* $Id: backup_conduit.c,v 1.39 2001/06/19 02:06:22 eskil Exp $ */
+/* $Id: backup_conduit.c,v 1.43 2001/11/05 10:03:22 eskil Exp $ */
 
 #include <glib.h>
 #include <gnome.h>
@@ -49,15 +49,26 @@
 #define DEBUG 1
 #define pi_mktag(c1,c2,c3,c4) (((c1)<<24)|((c2)<<16)|((c3)<<8)|(c4))
 #define MAXDBS 256
+#define cardno 0
+
 typedef struct file_db {
-        gchar name[256];
-        gchar dbname[256];
-        gint flags;
-        gulong creator;
-        gulong type;
+        gchar fname[256];
+	struct DBInfo info;
         gint maxblock;
+	gint entries;
 } file_db;
 
+static char*
+pi_unmktag (long type) {
+	static char tag[5];
+	tag[0] = (type >> 24) & 0xff;
+	tag[1] = (type >> 16) & 0xff;
+	tag[2] = (type >> 8) & 0xff;
+	tag[3] = type & 0xff;
+	tag[4] = 0;
+	return tag;
+}
+
 GnomePilotConduit *conduit_get_gpilot_conduit (guint32 pilotId);
 void conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit);
 
@@ -360,14 +371,22 @@ gnome_pilot_conduit_backup_create_name (GnomePilotConduitBackup *conduit,
 static gint
 gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 					GnomePilotDBInfo *dbinfo, 
-					ConduitCfg *cfg)
+					gpointer _cfg)
 {
 	char *name;
 	struct pi_file *f;
 	struct stat statb;
 	struct utimbuf times;
 	GList *iterator;
-	int result;
+	int result = 0;
+	ConduitCfg *cfg = (ConduitCfg*)_cfg;
+	int index;
+	int keep_reading;
+	int entries;
+	int err;
+	unsigned char *buffer;
+	PilotRecord remote;
+	int wrote;
 
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (dbinfo != NULL, -1);
@@ -377,7 +396,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 	for (iterator = cfg->exclude_files; iterator; iterator = g_list_next (iterator)) {
 		if( ! g_strcasecmp(iterator->data, PI_DBINFO (dbinfo)->name) ) {
 			g_message("excluded %s",PI_DBINFO (dbinfo)->name);
-			result = 1;
+			result = 2;
 			goto exit;
 		}
 	}
@@ -430,130 +449,138 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 		goto exit_and_free;
 	}
 
-        {
-		int index = 0;
-		int keep_reading = 1;
-		int num_records;
-		PilotRecord remote;	       	      
-		unsigned char *buffer;
-		int err;
+	err = dlp_OpenDB (dbinfo->pilot_socket, 
+			  cardno,
+			  dlpOpenRead,
+			  PI_DBINFO (dbinfo)->name,
+			  &dbinfo->db_handle);
+	
+	if (err < 0) {
+		g_warning ("error (%s) in opening '%s'", dlp_strerror (err), PI_DBINFO (dbinfo)->name);
+		result = -1;
+		goto db_close;
+	}
 
-		err = dlp_OpenDB (dbinfo->pilot_socket, 0,
-				  dlpOpenRead,
-				  PI_DBINFO (dbinfo)->name,
-				  &dbinfo->db_handle);
-			    
-		if (err < 0) {
-			g_warning ("error (%d) in opening '%s'", err, PI_DBINFO (dbinfo)->name);
-			result = -1;
-			goto close;
-		}
-
-		err = dlp_ReadOpenDBInfo (dbinfo->pilot_socket, 
-					  dbinfo->db_handle,
-					  &num_records);
+	err = dlp_ReadOpenDBInfo (dbinfo->pilot_socket, 
+				  dbinfo->db_handle,
+				  &entries);
 				    
-		if (err < 0) {
-			g_warning ("error (%d) in reading '%s'", err, PI_DBINFO (dbinfo)->name);
-			result = -1;
-			goto close;
-		}		
+	if (err < 0) {
+		g_warning ("error (%s) in reading '%s'", dlp_strerror (err), PI_DBINFO (dbinfo)->name);
+		result = -1;
+		goto db_close;
+	}		
 
-		buffer = g_new0 (unsigned char, 0xffff);
-		remote.buffer = buffer;
+	buffer = g_new0 (unsigned char, 0xffff);
+	remote.buffer = buffer;
 
-		do {
-			unsigned long type;
-			int id;
+	{
+		int l;
+		l = dlp_ReadAppBlock (dbinfo->pilot_socket,
+				      dbinfo->db_handle,
+				      0,
+				      buffer,
+				      0xffff);
+		if (l>0) {
+			pi_file_set_app_info (f, buffer, l);
+		}
+	}
+	
+
+	index = 0;
+	keep_reading = 1;
+	wrote = 0;
+	do {
+		unsigned long type;
+		int id;
 			
-			if (PI_DBINFO (dbinfo)->flags & dlpDBFlagResource) {
-				keep_reading = (dlp_ReadResourceByIndex (dbinfo->pilot_socket, 
-									 dbinfo->db_handle,
-									 index, 
-									 remote.buffer,
-									 &type,
-									 &id,
-									 &remote.len) >= 0);
-				/*
-				g_message ("appended resource %d (size %d), index %d/%d",
-					   id, remote.len,
-					   index, num_records);
-				*/
-				if (keep_reading) {				
-					err = pi_file_append_resource (f, 
-								       remote.buffer, 
-								       remote.len, 
-								       type,
-								       id);
-					if (err < 0) {
-						g_warning ("error in writing to file");
-					}
+		if (PI_DBINFO (dbinfo)->flags & dlpDBFlagResource) {
+			keep_reading = (dlp_ReadResourceByIndex (dbinfo->pilot_socket, 
+								 dbinfo->db_handle,
+								 index, 
+								 remote.buffer,
+								 &type,
+								 &id,
+								 &remote.len) >= 0);
+#ifdef DEBUG_REC_IO
+			g_message ("read resource %d, type = %s, size %d, , index %d/%d",
+				   id, pi_unmktag (type), remote.len,
+				   index, entries);
+#endif
+			if (keep_reading) {				
+				err = pi_file_append_resource (f, 
+							       remote.buffer, 
+							       remote.len, 
+							       type,
+							       id);
+				if (err < 0) {
+					g_warning ("error in writing to file");
+				} else {
+					wrote++;
+#ifdef DEBUG_REC_IO
+					g_message ("write resource %d, type = %s, size %d, index %d/%d",
+						   id, pi_unmktag (type), remote.len,
+						   index, entries);
+#endif
 				}
-			} else {
-				keep_reading = (dlp_ReadRecordByIndex (dbinfo->pilot_socket, 
-								       dbinfo->db_handle,
-								       index, 
-								       remote.buffer,
-								       &remote.recID,
-								       &remote.len,
-								       &remote.flags,
-								       &remote.catID) >= 0);
-				/*
-				g_message ("appended record %d (size %d), index %d/%d",
-					   remote.recID, remote.len,
-					   index, num_records);
-				*/
-				if (keep_reading) {
-					err = pi_file_append_record (f, 
-								     remote.buffer, 
-								     remote.len, 
-								     remote.flags,
-								     remote.catID,
-								     remote.recID);
-					if (err < 0) {
-						g_warning ("error in writing to file");
-					}
-				}
-				
-			}			
-			index++;
-			gnome_pilot_conduit_send_progress (GNOME_PILOT_CONDUIT (conduit),
-							   num_records,
-							   index);
-		} while (keep_reading && index < num_records);				  
+			}
+		} else {
+			keep_reading = (dlp_ReadRecordByIndex (dbinfo->pilot_socket, 
+							       dbinfo->db_handle,
+							       index, 
+							       remote.buffer,
+							       &remote.recID,
+							       &remote.len,
+							       &remote.flags,
+							       &remote.catID) >= 0);
 
-		g_free (buffer);
-		
-		/* dlp_ResetSyncFlags (dbinfo->pilot_socket, dbinfo->db_handle); */
-		dlp_CloseDB (dbinfo->pilot_socket, dbinfo->db_handle);
-
-		result = 0;
-	}
-
-#if 0
-	if(pi_file_retrieve (f, dbinfo->pilot_socket, 0) != 0) {
-		/* failed!? */
-		pi_file_close (f);
-		g_warning(_("Backup of %s failed!"),PI_DBINFO (dbinfo)->name);
-		gnome_pilot_conduit_send_error (GNOME_PILOT_CONDUIT(conduit),
-						_("Backup of %s failed!"),
-						PI_DBINFO (dbinfo)->name);
-		return -1;
-	} else {
-		/* ok */
-		pi_file_close (f);
-		times.actime = PI_DBINFO (dbinfo)->createDate;
-		times.modtime = PI_DBINFO (dbinfo)->modifyDate;
-		utime (name, &times);
-		return 0;
-	}
+#ifdef DEBUG_REC_IO
+			g_message ("read record %d, size %d, index %d/%d",
+				   remote.recID, remote.len,
+				   index, entries);
 #endif
 
- close:
+			if (keep_reading) {
+				err = pi_file_append_record (f, 
+							     remote.buffer, 
+							     remote.len, 
+							     remote.flags,
+							     remote.catID,
+							     remote.recID);
+				if (err < 0) {
+					g_warning ("error in writing to file");
+				} else {
+					wrote++;
+#ifdef DEBUG_REC_IO
+					g_message ("write record %d, size %d, index %d/%d",
+						   id, remote.len,
+						   index, entries);
+#endif
+				}
+
+			}
+				
+		}			
+		index++;
+		gnome_pilot_conduit_send_progress (GNOME_PILOT_CONDUIT (conduit),
+						   entries,
+						   index);
+	} while (keep_reading && index < entries);				  
+
+	g_message ("Wrote %d of %d %s, which is %s",
+		   wrote, entries, PI_DBINFO (dbinfo)->flags & dlpDBFlagResource ? "resources" : "records",
+		   wrote == entries ? "good" : "BAD");
+
+	g_free (buffer);
+		
+	//dlp_ResetSyncFlags (dbinfo->pilot_socket, dbinfo->db_handle); 		
+
 	pi_file_close (f);
 	times.actime = PI_DBINFO (dbinfo)->createDate;
 	times.modtime = PI_DBINFO (dbinfo)->modifyDate;
 	utime (name, &times);
+ db_close:
+	dlp_CloseDB (dbinfo->pilot_socket, dbinfo->db_handle);
  exit_and_free:
 	g_free (name);
  exit:	
@@ -564,11 +591,11 @@ static int
 compare(file_db *d1, file_db *d2)
 {
 	/* types of 'appl' sort later then other types */
-	if(d1->creator == d2->creator) {
-		if(d1->type != d2->type) {
-			if(d1->type == pi_mktag('a','p','p','l'))
+	if(d1->info.creator == d2->info.creator) {
+		if(d1->info.type != d2->info.type) {
+			if(d1->info.type == pi_mktag('a','p','p','l'))
 				return 1;
-			if(d2->type == pi_mktag('a','p','p','l'))
+			if(d2->info.type == pi_mktag('a','p','p','l'))
 				return -1;
 		}
 	}
@@ -581,7 +608,9 @@ static gint
 gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit, 
 					 int psock, 
 					 char *src_dir,
-					 ConduitCfg *cfg)
+					 GnomePilotConduitBackupRestore func,
+					 gpointer func_data,
+					 gpointer _cfg)
 {
 	DIR *dir;
 	struct dirent *dirent;
@@ -591,6 +620,9 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 	int i, j, max, size;
 	struct pi_file *file;
 	char *source;
+	int result = 0;
+	int reset = 0;
+	ConduitCfg *cfg = (ConduitCfg*)_cfg;
 
 	if (src_dir == NULL) {
 		source = cfg->backup_dir;
@@ -604,6 +636,9 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 
 	db = g_new0 (file_db*, MAXDBS);
 
+	gnome_pilot_conduit_send_message (GNOME_PILOT_CONDUIT (conduit), _("Collecting restore information..."));
+
+	/* Load all files in directory */
 	while( (dirent = readdir(dir)) ) {
 
 #ifdef DEBUG
@@ -615,28 +650,28 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 			continue;
 
 		db[dbcount] = g_new0 (file_db, 1);
-		g_snprintf (db[dbcount]->name, 255, "%s/%s", source, dirent->d_name);
+		g_snprintf (db[dbcount]->fname, 255, "%s/%s", source, dirent->d_name);
 
-		file = pi_file_open (db[dbcount]->name);
+		file = pi_file_open (db[dbcount]->fname);
 		if (file == 0) {
 #ifdef DEBUG
-			printf("backup_conduit: Unable to open '%s'!\n", db[dbcount]->name);
+			printf("backup_conduit: Unable to open '%s'!\n", db[dbcount]->fname);
 #endif
 			/* Skip this file */
 			continue;
 
 		}
 
-		pi_file_get_info (file, (PI_DBINFO (&info)));
+		/* Load db info */
+		pi_file_get_info (file, &db[dbcount]->info);
 
-		g_snprintf (db[dbcount]->dbname, 255, "%s", PI_DBINFO (&info)->name);
-		db[dbcount]->creator = PI_DBINFO (&info)->creator;
-		db[dbcount]->type = PI_DBINFO (&info)->type;
-		db[dbcount]->flags = PI_DBINFO (&info)->flags;
 		db[dbcount]->maxblock = 0;
 
-		pi_file_get_entries(file, &max);
-		for (i = 0; i < max; i++) {
+		/* Get number of records */
+		pi_file_get_entries(file, &db[dbcount]->entries);
+
+		/* Find biggest record */
+		for (i = 0; i < db[dbcount]->entries; i++) {
 			if (PI_DBINFO (&info)->flags & dlpDBFlagResource)
 				pi_file_read_resource(file, i, 0, &size, 0, 0);
 			else
@@ -645,13 +680,14 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 			if (size > db[dbcount]->maxblock)
 				db[dbcount]->maxblock = size;
 		}
-
+			
 		pi_file_close (file);
 		dbcount++;
 	}
 
 	closedir(dir);
 
+	/* Sort db's */
 	for (i=0;i<dbcount;i++) {
 		for (j=i+1;j<dbcount;j++)
 			if (compare(db[i],db[j])>0) {
@@ -662,47 +698,13 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 	}
 
 	for (i=0;i<dbcount;i++) {
-
-		if ( dlp_OpenConduit(psock) < 0) {
-#ifdef DEBUG
-			fprintf(stderr,"backup_conduit: Exiting on cancel. All data not restored.\n");
-#endif
-			return -1;
+		int err = func (db[i]->fname, i + 1, dbcount, func_data);
+		if (err<0) {
+			result = err;
 		}
+        }
 
-		file = pi_file_open (db[i]->name);
-		if (file == NULL) {
-#ifdef DEBUG
-			fprintf (stderr,"backup_conduit: Unable to open '%s'!\n", db[i]->name);
-#endif
-			break;
-		}
-#ifdef DEBUG
-		fprintf (stdout,"backup_conduit: Restoring %s...\n", db[i]->name);
-		fflush (stdout);
-#endif
-		g_message(_("Restoring %s from %s... "), db[i]->dbname, db[i]->name);
-		gnome_pilot_conduit_send_message(GNOME_PILOT_CONDUIT(conduit),
-						 _("Restoring %s from %s... "), 
-						 db[i]->dbname, 
-						 db[i]->name);
-		if (pi_file_install(file, psock, 0)<0) {
-#ifdef DEBUG
-			printf("failed.\n");
-#endif
-			g_message(_("Restore of %s failed"),db[i]->dbname);
-			gnome_pilot_conduit_send_error(GNOME_PILOT_CONDUIT(conduit),
-						       _("Restore of %s failed"),
-						       db[i]->dbname);
-		} else {
-#ifdef DEBUG
-			printf("OK\n");
-#endif
-		}
-
-		pi_file_close(file);
-	}
-
+ exit_and_free:
 #ifdef DEBUG
 	fprintf (stderr,"backup_conduit: Restore done\n");
 #endif
@@ -712,7 +714,8 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 	}
 
 	g_free(db);
-	return 0;
+ exit:
+	return result;
 }
 
 /*
